#!/usr/bin/perl -wt
package Mithraen::OpenVZ::config;

#=============================================================================#
#			Read OpenVZ configuration			      #
#=============================================================================#
# (C) Denis Smirnov <mithraen@freesource.info)                    04 Nov 2007 #
#=============================================================================#
use 5.008;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

our $VERSION = '0.01';

sub new {
    my $self = {
        'name2veid' => {},
        'veid2name' => {}
    };
    bless $self;
    foreach ( glob("/etc/vz/names/*") ) {
        my $ve_name = $_;
        my $ve_id   = readlink($_);
        $ve_name =~ s/^.*\///;
        $ve_id =~ s/^.*\/(\d+)\.conf/$1/ or next;
        $self->{name2veid}->{$ve_name} = $ve_id;
        $self->{veid2name}->{$ve_id}   = $ve_name;
    }

    return $self;
}

sub read_beancounters($$) {
    my $this = shift;
    my $db   = shift;
    local *IN;
    open( IN, "< /proc/user_beancounters" )
      || die "Can't open /proc/user_beancounters";
    <IN>, <IN>;

    my $id;
    my %p = ();

    while (<IN>) {
        $_ =~ s/^ {5,8}//;
        if ( $_ =~ s/^(\d+):\s+// ) {
            $id = $1;
        }
        else {
            $_ =~ s/^\s+//;
        }
        my @param = split /\s+/, $_;
        next if $param[0] eq 'dummy';
        $db->user_beancounter_add(
            $id,       $param[0], $param[1], $param[2],
            $param[3], $param[4], $param[5]
        );
    }
    close(*IN);
}

sub ve_cfg_fix($$) {
    my $this = shift;
    my $veid = shift;
    if ( !defined($veid) ) {
        $this->ve_cfg_fix($_) foreach sort keys %{ $this->{veid2name} };
    }
    else {
        return if $veid == 0;
        return unless $veid =~ /^(\d+)$/;
        $veid = $1;
        local %ENV;
        my $t = `/usr/sbin/vzcfgvalidate -r /etc/vz/conf/$veid.conf 2>&1`;
        print $t if $? != 0;
    }
}

1;
